
	processor 6502
	include ..\common\vcs.h
	

;============================================================================
; C O N S T A N T S
;============================================================================
NUMGRP					=	$07		; number of graphics/platform levels
GRPHEIGHT				=	$14		; height of each girder
PLAYER_HEIGHT			=	$12		; height of the climber
MAXSELECTLEVEL			=	$01		; maximum possible selected level

XMIN						=	$02		; minimum x value for players
XMAX						=	$95		; maximum x value for players

YMIN						=	$18		; minimum y value for players	
YMAX						=	$A8		; maximum y value for players

INITIAL_RANDOM_SEED	=	$B4A4		; good ol' BSC number :)

SELECT_DELAY			=	$3F

GAME_SOUND_VOLUME		=	$A6

STARTING_LIVES			=	$02
STARTING_LEVEL			=	$01

CLIMBER_X_START		=	XMAX-4
CLIMBER_Y_START		=	$A8

GIRDER_GRAPHIC			=	#%11000000

MAX_LEVEL				=	$06

RESERVE_LIVES_HEIGHT	=	$08
;----------------------------------------------------------------------------
; ladder graphic values
;
ODD_LADDER_PF1			=	%11000000
ODD_LADDER_PF2			=	%00001100

EVEN_LADDER_PF1		=	%00000110
EVEN_LADDER_PF2		=	%10000000

;----------------------------------------------------------------------------
; NTSC frame constants
;
NTSC_VBLANK_TIME		=	$26			; ((33 scanlines * 76 cycles) - 14) / 64 = ;$29
NTSC_OVERSCAN_TIME	=	$1D			; ((26 scanlines * 76 cycles) - 14) / 64 $23
NTSC_KERNEL_LINES		=	$C0

;----------------------------------------------------------------------------
; PAL frame constants
;
PAL_VBLANK_TIME		=	$45			; ((59 scanlines * 76 cycles) - 14) / 64;$34;$35
PAL_OVERSCAN_TIME		=  $3A			; ((50 scanlines * 76 cycles) - 14) / 64;$4E;$53
PAL_KERNEL_LINES		=	$E4

;----------------------------------------------------------------------------
; game state options
;
GAMERUNNING				=  #%00000000
TITLESCREEN				=  #%10000000
GAMEPAUSED				=  #%01000000
GAME_OVER_STATE		=	#%00000010
START_NEW_GAME			=	#%00000001
TRAINING_MODE			=	#%10000000

;----------------------------------------------------------------------------
; game speed
;
NTSC_LEVEL_1_SPEED	=	$05
NTSC_LEVEL_2_SPEED	=	$04
NTSC_LEVEL_3_SPEED	=	$03
NTSC_LEVEL_4_SPEED	=	$02
NTSC_LEVEL_5_SPEED	=	$01
NTSC_LEVEL_6_SPEED	=	$00

PAL_LEVEL_1_SPEED		=	NTSC_LEVEL_1_SPEED+1
PAL_LEVEL_2_SPEED		=	NTSC_LEVEL_2_SPEED+1
PAL_LEVEL_3_SPEED		=	NTSC_LEVEL_3_SPEED+1
PAL_LEVEL_4_SPEED		=	NTSC_LEVEL_4_SPEED
PAL_LEVEL_5_SPEED		=	NTSC_LEVEL_5_SPEED
PAL_LEVEL_6_SPEED		=	NTSC_LEVEL_6_SPEED

;----------------------------------------------------------------------------
; color constants
;
BLACK						=  $00
NTSC_LTBLUE				=  $78
NTSC_LADDER_COLOR		=	$99
NTSC_BALL_COLOR		=	$57
NTSC_GIRDER_COLOR		=	$5C
NTSC_BLUE				=  $86
NTSC_PEACH				=	$3C
NTSC_GREEN				= 	$DF
NTSC_RED					=	$40
NTSC_PINK				=	$3F;$4A
NTSC_SKY_BLUE			=	$9C
NTSC_WOOD_BROWN		=	$F4
NTSC_BROWN				=	$F8
NTSC_WHITE				=	$0F
NTSC_ATARI_AGE_BLUE	=	$84

PAL_BLACK				=	BLACK
PAL_LTBLUE				=	$BA
PAL_LADDER_COLOR		=	$99
PAL_BALL_COLOR			=	$AA
PAL_GIRDER_COLOR		=	$8C
PAL_BLUE					=	$B6
PAL_PEACH				=	$4C
PAL_GREEN				=	$5A
PAL_RED					=	$64
PAL_PINK					=	$6E
PAL_SKY_BLUE			=	$BC
PAL_WOOD_BROWN			=	$20
PAL_WHITE				=	NTSC_WHITE
PAL_BROWN				=	$24
PAL_ATARI_AGE_BLUE	=	$D4

;----------------------------------------------------------------------------
; joystick mask values
;
DOWN_MOTION_MASK		=	%00100000
UP_MOTION_MASK			=	%00010000

;----------------------------------------------------------------------------
; font heights
;
AA_FONT_HEIGHT			=	$0F
PRESENTS_FONT_HEIGHT	=	$08
CLIMBER_FONT_HEIGHT	=	$20
NUMBER_5_HEIGHT		=	$06
COPYRIGHT_HEIGHT		=	$08
TEXT_HEIGHT				=	$05
GAME_TEXT_HEIGHT		=	$05;$08

;----------------------------------------------------------------------------
; fine horizontal movement constants
;
HMOVE_L7					=	$70
HMOVE_L6					=	$60
HMOVE_L5					=	$50
HMOVE_L4					=	$40
HMOVE_L3					=	$30
HMOVE_L2					=	$20
HMOVE_L1					=	$10
HMOVE_0					=	$00
HMOVE_R1					=	$F0
HMOVE_R2					=	$E0
HMOVE_R3					=	$D0
HMOVE_R4					=	$C0
HMOVE_R5					=	$B0
HMOVE_R6					=	$A0
HMOVE_R7					=	$90
HMOVE_R8					=	$80

;============================================================================
; Z P - V A R I A B L E S
;============================================================================
   SEG.U variables
   org $80
   
; There is only 128 bytes of RAM available to the programmer. Locations
; $80 - $FF are available for RAM use.
;

; The random bytes are used to produce random number for the game. These
; variables and random routine are courtesy of Andrew Davie. He used it in
; his 'Qb' game. The source for 'Qb' can be found at http://www.atari2600.org.
;
random               ds 2

gameSelection        ds 1           ; the level number the player selected
gameState            ds 1
lives                ds 1           ; number of lives available to the player

backgroundColor      ds 1
platformColor        ds 1
ladderColor          ds 1
verPosP0             ds 1           ; vertical position of climber
horPosP0             ds 1           ; horizontal position of climber
attractMode          ds 1
horPosP1             ds NUMGRP
girderGraphics			ds NUMGRP
startCount           ds NUMGRP      ; count down for obstacle to move
playerMotion         ds 1		      ; speed of players
climberMotion			ds	1				; speed of climber
level                ds 1           ; current level being played
playerReflect			ds	1				; holds the reflect value for the climber

frameCount           ds 1           ; updated each frame
selectDebounce       ds 1           ; debounce flag for the select switch
temp                 ds 1
fireButtonDebounce	ds	1
playerState				ds	1
soundCounter			ds	1
soundIndex				ds	1
gameSpeed				ds	1
walkingSound			ds	1
gameClock				ds	1
playerPointer			ds	2
playerColorPointer	ds	2
duration					ds	1
walkingSoundIndex		ds	1
overlay					ds	27				; overlay place holder (thank you Andrew)


	org overlay								;[27/27]
;
; the following are needed for the game kernel
;
groupCount           ds 1           ; keeps track of which group of obstacles
                                    ; is being processed during the kernel
playerGraphics       ds PLAYER_HEIGHT
playerColors         ds PLAYER_HEIGHT
playerGraphic        ds 1
playerColor          ds 1
playerBufferedGraph	ds	1
playerBufferedColor	ds	1
vecKernel				ds	2

	org overlay								;[19/27]
;
; the following are needed for the title kernel
;
groupCount				ds	1
fontHeight				ds	1
graphicGroupPointer	ds	2
graphicsPointers		ds 12
fontColorPointer		ds	2
loopCount				ds	1

	org overlay								;[5/27]
;
; the following are needed for the game calculations
;
allowedMotion			ds	1
ladderPointer			ds 4


	org overlay								;[16/27]
;
; the following are needed for the status kernel
;
livesPointer			ds	4
levelPointer			ds	12
	
	SEG code
	org $F000       ; 4K to burn 2732 ROM

Start
;
; The 2600 powers up in a completely random state, except for the PC which
; is set to the location at $FFC.  Therefore the first thing we must do is
; to set everything up inside the 6502.
;
;   sei                 ; The 6507 has no interrupt capability. This is here
								; for clarity purposes and commented out to save 1
								; byte :)
   cld                  ; Clear BCD bit
   ldx #$FF
   txs                  ; Set stack to the beginning
     
   lda #$00
;
; The 2600 has 128 bytes of RAM. This routine will not clear VSYNC
;
.clearRAM
   sta $00,x
   dex
   bne .clearRAM
   
;----------------------------------------------------------GameInitialization
;
; Initialize the game variables at cart start up.
;
GameInitialization SUBROUTINE
	ldx #$03
.initializationLoop
	lda InitializationTable,x
	sta random,x
	dex
	bpl .initializationLoop
	
	lda #NTSC_LADDER_COLOR
	bit SWCHB
	bpl .setLadderColor
	lda #PAL_LADDER_COLOR
.setLadderColor
	sta ladderColor
	
	lda #%00010001
	sta CTRLPF
	
	lda #$01
	sta playerMotion
	
	jsr SetGameSpeed

MainLoop	
;----------------------------------------------------------------VerticalSync
;
; This routine will take care of vertical sync house keeping. Vertical sync
; starts a new television frame. Each frame starts with 3 vertical sync
; lines. These signal to the television to start a new frame. During
; the vertical sync time console switches will be checked. The right
; difficulty switch will determine if the game should use the NTSC or PAL
; color palette.
;
VerticalSync SUBROUTINE
	ldx #NTSC_VBLANK_TIME
	bit SWCHB								; check the right difficulty switch
	bpl .startVerticalSync				; if set to B then use NTSC settings
	ldx #PAL_VBLANK_TIME					; else use PAL settings
.startVerticalSync
	ldy #$00
	lda #$02
	sta WSYNC								; make sure VSYNC happens on a new line
	sta VSYNC								; start a new frame
	sta WSYNC								; first line for VSYNC
	sta WSYNC								; second line of VSYNC
	sta WSYNC								; third and final line of VSYNC
	sty VSYNC
	stx TIM64T
	
;--------------------------------------------------------CheckConsoleSwitches
;
; Check the VCS's console switches. The right difficulty switch will determine
; if the game should use the NTSC or PAL color palette.
;
CheckConsoleSwitches SUBROUTINE
	inc frameCount							; frame count is update every frame
	bne .skipAttractMode					;
	lda attractMode						; if already in attract mode then skip
	bmi .skipAttractMode					; incrementing the attract mode counter
	inc attractMode						; attract mode updated every 255 frames
												; until it reaches #$80
.skipAttractMode
	lda SWCHB								; read the console switch value
	lsr										; move the reset value to the carry bit

	bcs .skipReset							; reset button not pressed
	lda #START_NEW_GAME
	sta gameState
	ldy #$00
	jmp .enableTIA							; reset button press

.skipReset
	tay										; move console switch value for later use
	lda frameCount							; 
	and #SELECT_DELAY						; the select switch is checked ~ every 60
	bne .checkSelectSwitch				; frames or ~ every second
	sta selectDebounce					;
.checkSelectSwitch
	tya										; get the console switch value
	lsr										; move the select value to the carry bit
	bcs .continueConsoleCheck			; select switch not down so continue
												; vertical sync
.selectSwitchDown
	bit selectDebounce					; if bit 7 is set high then ignore the
	bmi .continueConsoleCheck			; select switch value
.continueSelectSwitchDown
	lda #$FF									; show that the select switch is held
	sta selectDebounce					; for the next frame
	ldy #$00									; to be used to clear the attract timer
	lda attractMode						; 
	sty attractMode						; clear the attract mode timer
	sty frameCount
	bmi .continueConsoleCheck			; if currently in attract mode continue
												; with vertical sync
	bit gameState							; if currently on the title screen then
	bmi .incrementGameSelection		; increase the game selection
	
	lda #TITLESCREEN						; game select was pressed and recorded so
	sta gameState							; put the user back to the title screen
	jmp .continueConsoleCheck			; the game selection is only increased
												; while on the title screen so continue
												; vertical sync
.incrementGameSelection
   lda gameSelection						; determine if the game selection should
   eor #TRAINING_MODE
   sta gameSelection						; set to #$00
  
.continueConsoleCheck
	lda gameState
	cmp #GAME_OVER_STATE
	beq .endConsoleSwitchCheck
	
	lda frameCount							; check the fire button ~every 60 seconds
	and #SELECT_DELAY
	bne .checkFireButton
	jmp .checkJoystick					; the joystick is checked to determine if
												; the attract mode should be turned off
.checkFireButton
	ldy #$00									; used to reset the attract mode timer
	bit INPT4								; bit 7 of INPT4 is set high so the fire
	bmi .clearFireButtonDebounce		; button is not depressed

;
; The fire button was pressed. Check to see if it was held down. If so then
; ignore the button press. If the button press is valid (it wasn't held down
; and enough time has elapsed) then set the game state accordingly.
;

	lda #$FF									; check if the fire button is down from
	cmp fireButtonDebounce				; the previous frame
	beq .checkJoystick					;
	sta fireButtonDebounce				; show the fire button was depressed
	lda gameState							; start a new game if the user is on the
	bpl .checkPauseState					; title screen
	
	lda #START_NEW_GAME
	sta gameState
	bne .continueConsoleSwitchCheck
	
.clearFireButtonDebounce
	inc fireButtonDebounce				; show the fire button was not pressed
	
.checkJoystick
	lda SWCHA								; if the joystick was moved turn off
	cmp #$FF									; attract mode (enable TV output)
	bne .enableTIA
	beq .endConsoleSwitchCheck
	
.checkPauseState
	bit attractMode						; the fire button was pressed clear the
	bmi .enableTIA							; attract mode
	
	eor #GAMEPAUSED						; EOR will switch the bits to set the
	sta gameState							; game mode
	
.continueConsoleSwitchCheck
	sty attractMode						; clear the attract mode timer
	sty frameCount
	
.endConsoleSwitchCheck

	lda gameState							; get the current game state
	cmp #GAME_OVER_STATE
	bne .checkForNewGameState
	lda attractMode
	bmi .showTitleScreen
	jmp DoneGameCalculations
.showTitleScreen
	lda #TITLESCREEN
	sta gameState
	bmi TitleScreenCalculations
;	jmp DoneGameCalculations
.checkForNewGameState
	cmp #START_NEW_GAME
	bne .enableTIA

	jsr StartNewGame
.enableTIA
	bit gameState
	bpl GameCalculations					; game is in progress

;-----------------------------------------------------TitleScreenCalculations
;
; Set up the players for a 6 digit display. They will be centered on the
; screen for the title screen information.
; 37
TitleScreenCalculations SUBROUTINE
	ldx #$06
	jsr RandomByte
	sta WSYNC
.coarseMovePlayers
	dex
	bpl .coarseMovePlayers
	nop
	sta RESP0 ;39
	sta RESP1 ;42
	lda #HMOVE_R3							; move right 3 pixels 42
	sta HMP0
	lda #HMOVE_R2							; move right 2 pixels 44
	sta HMP1
	sta WSYNC
	sta HMOVE
	
	lda #NTSC_SKY_BLUE
	ldx #NTSC_WOOD_BROWN
	bit SWCHB								; check the right difficulty switch
	bpl .setTitleScreenColors			; if set to B then use NTSC settings
	lda #PAL_SKY_BLUE						; else use PAL settings
	ldx #PAL_WOOD_BROWN
.setTitleScreenColors
	sta COLUBK
	stx COLUPF
	
	lda #>TitleDisplayKernel
	sta vecKernel+1
	lda #<TitleDisplayKernel
	sta vecKernel
	
	lda #$03
	sta NUSIZ0								; 3 copies of GRP0 close
	sta NUSIZ1								; 3 copies of GRP1 close
	
	sta VDELP0								; vertical delay for GRP0
	sta VDELP1								; vertical delay for GRP1
	
	sta HMCLR								; clear horizontal movement
	lda #$00
	sta attractMode						; reset the attract mode
	sta REFP0								; set the players not to reflect so the
	sta REFP1								; title screen displays properly
	jmp DisplayKernel

;------------------------------------------------------------GameCalculations
;
; The players will be moved in this routine. The sound will be done during
; overscan calculations.
;
GameCalculations	SUBROUTINE
	bvc .decrementGameSpeed				; if the game is not paused then move the
												; players
	jmp DoneGameCalculations
	
.decrementGameSpeed
	dec gameSpeed
	bmi .incrementGameClock
	jmp DoneGameCalculations
.incrementGameClock
	jsr SetGameSpeed	
	inc gameClock
;
; Move the obstacles horizontally. This routine is used courtesy of
; Piero Cavina. It is based on his PCMSD demo which eventually became
; Oyston. His original PCMSD source can be found at
; http://www.biglist.com/lists/stella/archives/199704/msg00015.html
;
.moveGirders
	lda #$00
	sta attractMode						; reset the attract mode
	ldx #NUMGRP-1
.moveGirdersLoop
	lda startCount,x						; if the start count is zero then move
	beq .moveGirder						; the corresponding girder
	dec startCount,x						; decrement the start count until it
	jmp .checkNextGroup					; reaches zero
.moveGirder
	lda horPosP1,x							; get girder's horizontal position
	cmp #XMAX								; if the girder hasn't reached right edge
	bcc .moveGirderRight					; then continue moving right
	jsr RandomByte							; get a random start count for the girder
	sta startCount,x
	lda #XMIN								; wrap girder around to the left side of
	bne .setGirderPosition				; the screen
.moveGirderRight
	clc										; move the girder playermotion pixels to
	adc playerMotion						; the right
.setGirderPosition
	sta horPosP1,x
.checkNextGroup
	dex
	bpl .moveGirdersLoop
	
	bit playerState						; check the current player state if not
	bpl CheckJoystickValues				; doing death animation check joysticks
;
; death animation
;

	lda allowedMotion						; if the climber is not a walkway he is
	and #DOWN_MOTION_MASK				; descended until he hits the walkway
	bne .doDeathAnimation
	lda verPosP0
   clc
   adc climberMotion
   sta verPosP0
.doDeathAnimation
	lda gameClock
	lsr
	lsr
	and #$01
	tax
	lda LowDeathAnimationTable,x
	sta playerPointer
	lda HighDeathAnimationTable,x
	sta playerPointer+1
	
	ldx #$00
	bit SWCHB
	bpl .loadDeathColors
	ldx #$02
.loadDeathColors
	ldy DeathAnimationColorTable,x
	lda DeathAnimationColorTable+1,x
	jmp .setVerticalColorPointer
	
CheckJoystickValues
	ldx #$00
	bit lives
	bpl .checkAllowedMotion
	
	lda #$FF
	sta allowedMotion
.checkAllowedMotion
	lda allowedMotion						; allowedMotion was determined during overscan
   bmi .checkForLeftJoystickMove
   
   lda horPosP0							; if the climber has reached his right
   cmp #CLIMBER_X_START					; most edge then don't move him
   bcc .moveClimberRight
	bcs .doneHorizontalCalculations
	
.moveClimberRight
   clc
   adc climberMotion
   sta horPosP0
   lda #$0F
   sta playerReflect						; keep the reflect value of the climber
   bne .doneHorizontalCalculations
   
.checkForLeftJoystickMove
	bit allowedMotion
	bvs .checkForDownJoystickMove

   lda horPosP0							; if the climber has reached his left
   cmp #XMIN+5								; most edge then don't move him
   bcs .moveClimberLeft
   
   bcc .doneHorizontalCalculations
   
.moveClimberLeft
	sec
	sbc climberMotion
	sta horPosP0
	
	ldx #$00
	stx playerReflect
	beq .doneHorizontalCalculations
   
.checkForDownJoystickMove
;	ldx #$00
	lda allowedMotion
	and #DOWN_MOTION_MASK

	bne .checkForUpJoystickMove

   lda verPosP0							; if the climber has reached his lower
   cmp #YMAX								; bound then don't move him
;   bne .moveClimberDown
   beq DoneGameCalculations
   
.moveClimberDown
   clc
   adc climberMotion
   sta verPosP0
   bne .doneVerticalCalculations
   
.checkForUpJoystickMove
	lda allowedMotion
	and #UP_MOTION_MASK
	
	bne DoneGameCalculations
	
	lda verPosP0							; if the climber has reached his upper
	cmp #YMIN								; bound then don't move him
	beq DoneGameCalculations

.moveClimberUp
	sec
   sbc climberMotion
   sta verPosP0
   
.doneVerticalCalculations	
	ldx #$00
	stx playerReflect
	lda gameClock
	lsr
	lsr
	and #$01
	tax

	lda LowVerticalMovementTable,x
	sta playerPointer
	lda HighVerticalMovementTable,x
	sta playerPointer+1
	
	ldx #$00
	bit SWCHB
	bpl .loadVerticalColors
	ldx #$02

.loadVerticalColors
	ldy VerticalColorTable,x
	lda VerticalColorTable+1,x
	
.setVerticalColorPointer
	sta playerColorPointer+1
	sty playerColorPointer
	bne DoneGameCalculations
	
.doneHorizontalCalculations
	
	lda gameClock
	lsr

	and #$03
	tax
	
	lda LowHorizontalMovementTable,x
	sta playerPointer
	lda HighHorizontalMovementTable,x
	sta playerPointer+1
	
	ldx #$00
	bit SWCHB
	bpl .setHorizontalColorPointer
	ldx #$02
.setHorizontalColorPointer
	lda HorizontalColorTable,x
	sta playerColorPointer
	lda HorizontalColorTable+1,x
	sta playerColorPointer+1
	jmp .checkForDownJoystickMove
   
DoneGameCalculations	SUBROUTINE
	lda playerReflect
	sta REFP0
	ldy #PLAYER_HEIGHT-1
.loadPlayerGraphics
	lda (playerPointer),y
	sta playerGraphics,y
	lda (playerColorPointer),y
	sta playerColors,y
	dey
	bpl .loadPlayerGraphics
	
.positionClimber
	sta WSYNC
	sta HMOVE
	
	lda #NUMGRP
	sta.w groupCount
	lda.w horPosP0
	bit $00									; waste 3 cycles
	sec
.coarseMoveClimber
	sbc #$0F
	bcs .coarseMoveClimber
	sta RESP0
	
	sta WSYNC
;-------------------------
	sta HMOVE
	tay
	lda HorizontalMoveTable-240,y
	
	ldy #$00
	
	sty NUSIZ0
	sty NUSIZ1
	sta RESBL
	sty VDELP0
	sty playerGraphic
	sty playerColor
	sta HMP0
	lda #HMOVE_R3
	sta HMBL

	sta WSYNC
;-------------------------
	sta HMOVE
	tya
;
; This a modified version of Thomas' skipDraw. x is set during VBLANK as an
; offset for the player. During the kernel x will be checked to see if the
; player needs to be drawn. This elimates the use of x in the game kernel
; for anything other than drawing the main player.
;
   sec
   sbc verPosP0
   adc #PLAYER_HEIGHT
   tax

	lda #NTSC_LADDER_COLOR
	ldy #NTSC_GIRDER_COLOR
	bit SWCHB								; check the right difficulty switch
	bpl .setStaticColors					; if set to B then use NTSC settings
	lda #PAL_LADDER_COLOR				; else use PAL settings
	ldy #PAL_GIRDER_COLOR
.setStaticColors
	sta ladderColor
	sta COLUPF
	sty COLUP1
	
	sta HMCLR
	
	lda #>StartGameKernel
	sta vecKernel+1
	lda #<StartGameKernel
	sta vecKernel
	
	ldy groupCount
	sty VDELP1
   
   
	lda backgroundColor
	sta COLUBK
	
;---------------------------------------------------------------DisplayKernel
;
; The screen must be updated each frame on the 2600. Here the program waits
; for vertical blank to end and then jumps to the appropriate display kernel
; (i.e. GameKernel or TitleDisplayKernel). These values were set during the
; vertical blank.
;
DisplayKernel SUBROUTINE
	ldy INTIM
	bne DisplayKernel
	
	sty WSYNC								; end the current scan line
;-------------------------
	sty HMOVE
	
	bit attractMode
	bpl .enableTIA
	ldy #$02
.enableTIA
	sty VBLANK								; turn on the TIA for output
	jmp (vecKernel)						; jump to the appropriate kernel
	
Overscan	SUBROUTINE
	lda #$02
	sta WSYNC								; end last scanline
	sta VBLANK								; turn of TIA

			
	lda #NTSC_OVERSCAN_TIME				; assume NTSC display ($1D)
	bit SWCHB								; check the right difficulty switch
	bpl .setOverscanTime					; if set to B then use NTSC settings
	lda #PAL_OVERSCAN_TIME				; else use PAL settings ($3A)
.setOverscanTime
	sta TIM64T

	lda gameState							; get the current game state
	beq DetermineClimberAllowedMotion
												; if the game isn't being played then
												; jump to the big wait
	jmp OverscanWait
DetermineClimberAllowedMotion	SUBROUTINE
	ldy #NUMGRP-1
	lda verPosP0
.determineWalkwayLoop
	cmp WalkwayValues,y
	beq .walkwayFound						; the climber is standing on a platform
	dey
	bpl .determineWalkwayLoop

	lda #%11011111
	bit playerState						; if the player was hit and no walkay was
	bmi .setAllowedMotion				; found make the player fall
	
	lda SWCHA								; don't allow horizontal movement but allow
	ora #%11000000							; vertical movement (climber is between
	bmi .setAllowedMotion				; platforms)
.walkwayFound
	lda #%11110000
	bit playerState
	bmi .setAllowedMotion
	and SWCHA								; allow horizontal movement
	
	sta allowedMotion
	
	tya										; y holds the walkway number
	and #%00000001							; mask all but bit 0
	asl										; multiply the value by 2
	tay										; this will now be used as an index for
												; odd/even walkways
	ldx #$00
.storeValidLadderValues
	lda LadderValuePointer,y
	sta ladderPointer,x
	iny
	inx
	cpx #$04
	bne .storeValidLadderValues
		
	ldy #$13
	lda horPosP0
.determineLadderNumber
	cmp (ladderPointer),y				; check and see if the climber's horizontal 
	beq .allowDownwardMotion			; position compared to the ladder values
	cmp (ladderPointer+2),y
	beq .allowUpwardMotion
	dey
	bpl .determineLadderNumber
	
	lda #%00110000							; don't allow vertical motion
	bne .setVerticalMotion
	
.allowUpwardMotion
	lda #%00100000
	bne .setVerticalMotion
	
.allowDownwardMotion
	lda #%00010000
	
.setVerticalMotion
	ora allowedMotion
	
.setAllowedMotion
	sta allowedMotion
	
CheckPlayerCollisions	SUBROUTINE
	lda CXP0FB								; player0/ball collision
	and #%01000000
	bne .playerBallCollision

	lda CXPPMM								; player0/player1 collision
	bmi .playerGirderCollision

	jmp OverscanWait
	
.playerGirderCollision
	tax
	lda #$FF
	and allowedMotion
	sta allowedMotion
	
	bit playerState
	bmi .playDeathSound
	
	stx playerState

	lda #$15
	sta soundIndex
	lda #$00
	sta soundCounter
	
	lda #$01
	sta duration

	
.playDeathSound
	dec duration
	beq .playNextDeathNote
	jmp OverscanWait
.playNextDeathNote	
	ldy #$0C
	lda #$0A
	sta duration
	ldx soundCounter
	inc soundCounter
	jsr PlaySound
	bne OverscanWait
	lda #$00
	sta playerState
	dec lives
	bpl .continueGame
	
	lda #GAME_OVER_STATE
	sta gameState
	lda #$7E
	sta attractMode
	lda #$1E
	sta frameCount
	bne .turnOffSound
	
.playerBallCollision
	bit playerState
	bvs .playSuccessSound
	
	sta playerState
	
	lda #$01
	sta duration
	
	lda #$06
	sta soundIndex
	
.playSuccessSound
	ldx #$FF
	stx allowedMotion
	
	ldy #$06
	dec duration
	bne OverscanWait
	
	lda #$0A
	sta duration
	
	lda soundIndex
	tax
	lda SuccessSoundTable,x
	
	jsr PlaySound
	bne OverscanWait
	
	lda gameSelection
	bpl .incrementLevel
	
	jsr SetGirderGraphics
	
	ldx #NUMGRP-1
.checkGirderGraphicsLoop
	lda girderGraphics,x
	beq .continueGame
	dex
	bpl .checkGirderGraphicsLoop
	
.incrementLevel
	lda backgroundColor					; change the background color to show the
	sec										; player they have advanced to the new level
	sbc #$4E
	sta backgroundColor
	
	lda platformColor
	adc #$3E
	sta platformColor
	
	clc
	sed
	lda level
	adc #$01
	sta level
	cld
	
	cmp #MAX_LEVEL
	bcc .continueGame

.continueGame
	jsr RestartLevel						; restart with the current level number
	
.turnOffSound
	lda #$00									; turn of the success sound
	ldx #$05
.turnOffSoundLoop
	sta AUDC0,x
	dex
	bpl .turnOffSoundLoop
	sta CXCLR

OverscanWait
	lda INTIM
	bne OverscanWait
	
	jmp MainLoop
	
;----------------------------------------------------------------StartNewGame
;
StartNewGame
	lda #STARTING_LIVES
	sta lives								; set the starting number of lives

	ldx #NUMGRP-1
	lda #$00
	
	sta selectDebounce
.resetGirderGraphicsLoop
	sta girderGraphics,x
	dex
	bpl .resetGirderGraphicsLoop

	tax
	sta backgroundColor
	
	lda #STARTING_LEVEL
	sta level
	
	jsr SetGirderGraphics
												; |
	lda #NTSC_PINK							; |
	bit SWCHB								; |- set the platform color depending on
	bpl .setPlatformColor				; |  if this for NTSC or PAL
	lda #PAL_PINK							; |
	ldx #$02									; |
.setPlatformColor							; |
	sta platformColor
	
	jsr RestartLevel						; restart the current level which will
												; be STARTING_LEVEL
	lda #GAMERUNNING
	sta gameState
	
	lda #$01
	sta climberMotion
	rts
	
;---------------------------------------------------------InitializationTable
;
InitializationTable
	.word INITIAL_RANDOM_SEED
	.byte #$01
	.byte #TITLESCREEN
	
;------------------------------------------------------------------RandomByte
;
; The VCS has no random routine so one must be written by the programmer.
;
; This routine assumes random was defined and reserved 2 bytes of storage.
; Before calling this routine random MUST be set to a non-zero number.It's
; recommended to use user input as a seed for this routine to get the "best"
; possible random number.
; Upon return from this function, the accumulator will hold a pseudo random
; number.
;
RandomByte SUBROUTINE
	lda random
	bne .skipInit
	lda #$FF
.skipInit:
	asl
	asl
	asl
	eor random
	asl
	rol random
	lda random
	rts
	
;----------------------------------------------------------------SetGameSpeed
;
SetGameSpeed
	lda level
	cmp #MAX_LEVEL
	bcc .setGameSpeedOffset
	
	ldy #MAX_LEVEL-1
	bne .setGameSpeed
	
.setGameSpeedOffset
	ldy level
	dey

.setGameSpeed	
	bit SWCHB
	bpl .setNTSCGameSpeed
	
	lda PALGameSpeedTable,y
	bpl .storeGameSpeed
.setNTSCGameSpeed
	lda NTSCGameSpeedTable,y
.storeGameSpeed
	sta gameSpeed
	rts

;----------------------------------------------------------NTSCGameSpeedTable
NTSCGameSpeedTable
	.byte #NTSC_LEVEL_1_SPEED
	.byte #NTSC_LEVEL_2_SPEED
	.byte #NTSC_LEVEL_3_SPEED
	.byte #NTSC_LEVEL_4_SPEED
	.byte #NTSC_LEVEL_5_SPEED
	.byte #NTSC_LEVEL_6_SPEED
	
;-----------------------------------------------------------PALGameSpeedTable
PALGameSpeedTable
	.byte #PAL_LEVEL_1_SPEED
	.byte #PAL_LEVEL_2_SPEED
	.byte #PAL_LEVEL_3_SPEED
	.byte #PAL_LEVEL_4_SPEED
	.byte #PAL_LEVEL_5_SPEED
	.byte #PAL_LEVEL_6_SPEED
	
;-----------------------------------------------------------SuccessSoundTable
;
; Music bytes for grabbing the baseball.
;
SuccessSoundTable
;	.byte $48,$79,$51,$79,$60,$79
	.byte $18,$19,$11,$19,$10,$19
	
;-----------------------------------------------------------------KernelHiTbl
KernelHiTbl
	.byte #<DrawEvenLadderGroup
	.byte #<DrawOddLadderGroup
	.byte #<DrawLadderGroup
	
KernelLoTbl
	.byte #>DrawEvenLadderGroup
	.byte #>DrawOddLadderGroup
	.byte #>DrawLadderGroup
	
;---------------------------------------------------------PlatformLadderValue
;
; This table holds the ladder graphic values for the platform. There is an
; entry for each group.
;
; Do not cross a page boundary. Timing is critical.
;
PF1PlatformLadderValue
	.byte $00
	.byte #EVEN_LADDER_PF1 | ODD_LADDER_PF1
	.byte #EVEN_LADDER_PF1 | ODD_LADDER_PF1
	.byte #EVEN_LADDER_PF1 | ODD_LADDER_PF1
	.byte #EVEN_LADDER_PF1 | ODD_LADDER_PF1
	.byte #EVEN_LADDER_PF1 | ODD_LADDER_PF1
	.byte #EVEN_LADDER_PF1
	.byte $00
	
PF2PlatformLadderValue
	.byte $00
	.byte #EVEN_LADDER_PF2 | ODD_LADDER_PF2
	.byte #EVEN_LADDER_PF2 | ODD_LADDER_PF2
	.byte #EVEN_LADDER_PF2 | ODD_LADDER_PF2
	.byte #EVEN_LADDER_PF2 | ODD_LADDER_PF2
	.byte #EVEN_LADDER_PF2 | ODD_LADDER_PF2
	.byte #EVEN_LADDER_PF2
	.byte $00
	
;----------------------------------------------------------------DivisibleBy4
;
; Look up table to decide if a value is divisible by 4. A zero in a possition
; means the value does not satisfy the condition. The y-register holds the
; value in question. Valid values for y are 20 >= y >= 0.
;
DivisibleBy4
	.byte $01,$00,$00,$00,$01
	.byte $00,$00,$00,$01,$00
	.byte $00,$00,$01,$00,$00
	.byte $00,$01,$00,$00,$00,$01
	
;------------------------------------------------------LadderAndWalkwayTables
;
; Tables used for the ladder and walkway computations
;
LadderValuePointer
	.byte #<OddLadderValues,#>OddLadderValues
	.byte #<EvenLadderValues,#>EvenLadderValues
	.byte #<OddLadderValues,#>OddLadderValues
	
EvenLadderValues
;	.byte $10,$11,$12,$13,$14,$38,$39,$3A,$3B,$3C
;	.byte $60,$61,$62,$63,$64,$88,$89,$8A,$8B,$8C
	.byte $07,$08,$09,$0A,$0B,$30,$31,$32,$33,$34
	.byte $57,$58,$59,$5A,$5B,$80,$81,$82,$83,$84

OddLadderValues
;	.byte $24,$25,$26,$27,$28,$4C,$4D,$4E,$4F,$50
;	.byte $74,$75,$76,$77,$78,$78,$78,$78,$78,$78
	.byte $1C,$1D,$1E,$1F,$20,$43,$44,$45,$46,$47
	.byte $6C,$6D,$6E,$6F,$70,$70,$70,$70,$70,$70
	
WalkwayValues
	.byte $18,$30,$48,$60,$78,$90,$A8
	
;-----------------------------------------------------------ClimberSpriteData
;
; The sprite data for the Climber object. Notice that they are not stored in
; the ROM upside down. The data is read in to ZP so there is no need for the
; data to be inverted. If the data was read directly from the table in the
; kernel then it would be better for the data to be inverted to save on cycle
; counts and ROM space.
;
ClimberSpriteData
HorizontalMoveFrame1   
	.byte $1C ;|   XXX  |
	.byte $3E ;|  XXXXX |   
	.byte $FE ;|XXXXXXX |
	.byte $3E ;|  XXXXX |
	.byte $5F ;| X XXXXX|
	.byte $7F ;| XXXXXXX|
	.byte $7F ;| XXXXXXX|
	.byte $3E ;|  XXXXX |
	.byte $1C ;|   XXX  |
	.byte $3F ;|  XXXXXX|
	.byte $F9 ;|XXXXX  X|
	.byte $3A ;|  XXX X |
	.byte $32 ;|  XX  X |
	.byte $3E ;|  XXXXX |
	.byte $36 ;|  XX XX |
	.byte $36 ;|  XX XX |
	.byte $12 ;|   X  X |
	.byte $36 ;|  XX XX |

HorizontalMoveFrame2   
	.byte $1C ;|   XXX  |
	.byte $3E ;|  XXXXX |   
	.byte $FE ;|XXXXXXX |
	.byte $3E ;|  XXXXX |
	.byte $5F ;| X XXXXX|
	.byte $7F ;| XXXXXXX|
	.byte $7F ;| XXXXXXX|
	.byte $3E ;|  XXXXX |
	.byte $1C ;|   XXX  |
	.byte $3E ;|  XXXXX |
	.byte $2B ;|  X X XX|
	.byte $23 ;|  X   XX|
	.byte $72 ;| XXX  X |
	.byte $7E ;| XXXXXX |
	.byte $6E ;| XX XXX |
	.byte $6F ;| XX XXXX|
	.byte $27 ;|  X  XXX|
	.byte $61 ;| XX    X|

HorizontalMoveFrame3	
	.byte $1C ;|   XXX  |
	.byte $3E ;|  XXXXX |   
	.byte $FE ;|XXXXXXX |
	.byte $3E ;|  XXXXX |
	.byte $5F ;| X XXXXX|
	.byte $7F ;| XXXXXXX|
	.byte $7F ;| XXXXXXX|
	.byte $3E ;|  XXXXX |
	.byte $9C ;|X  XXX  |
	.byte $FF ;|XXXXXXXX|
	.byte $FF ;|XXXXXXXX|
	.byte $3A ;|  XXX X |
	.byte $76 ;| XXX XX |
	.byte $7E ;| XXXXXX |
	.byte $2F ;| X XXXX |
	.byte $66 ;| XX  XX |
	.byte $07 ;|     XXX|
	.byte $03 ;|      XX|
	
VerticalMoveFrame1
	.byte $1C ;|   XXX  |
	.byte $3E ;|  XXXXX |
   .byte $3E ;|  XXXXX |
   .byte $3E ;|  XXXXX |
   .byte $BE ;|X XXXXX |
   .byte $DC ;|XX XXX  |
   .byte $FE ;|XXXXXXX |
   .byte $73 ;| XXX  XX|
   .byte $37 ;|  XX XXX|
   .byte $13 ;|   X  XX|
   .byte $7B ;| XXXX XX|
   .byte $F2 ;|XXXX  X |
   .byte $FE ;|XXXXXXX |
   .byte $FE ;|XXXXXXX |
   .byte $FE ;|XXXXXXX |
   .byte $6E ;| XX XXX |
   .byte $0E ;|    XXX |
   .byte $04 ;|     X  |
	
VerticalMoveFrame2
	.byte $38 ;|  XXX   |
	.byte $7C ;| XXXXX  |
   .byte $7C ;| XXXXX  |
   .byte $7C ;| XXXXX  |
   .byte $7D ;| XXXXX X|
   .byte $3B ;|  XXX XX|
   .byte $7F ;| XXXXXXX|
   .byte $CE ;|XX  XXX |
   .byte $DC ;|XX XXX  |
   .byte $C8 ;|XX  X   |
   .byte $EE ;|XXX XXX |
   .byte $4F ;| X  XXXX|
   .byte $7F ;| XXXXXXX|
   .byte $7F ;| XXXXXXX|
   .byte $7F ;| XXXXXXX|
   .byte $76 ;| XXX XX |
   .byte $70 ;| XXX    |
   .byte $20 ;|  X     |
   
DeathFrame1
	.byte $28 ;|  X X   |
	.byte $42 ;| X    X |
	.byte $14 ;|   X X  |
	.byte $1C ;|   XXX  |
	.byte $3E ;|  XXXXX |   
	.byte $FE ;|XXXXXXX |
	.byte $3E ;|  XXXXX |
	.byte $5F ;| X XXXXX|
	.byte $7F ;| XXXXXXX|
	.byte $7F ;| XXXXXXX| mustache
	.byte $3E ;|  XXXXX | chin
	.byte $1C ;|   XXX  | neck
	.byte $3F ;|  XXXXXX|
	.byte $F9 ;|XXXXX  X|
	.byte $3A ;|  XXX X |
	.byte $32 ;|  XX  X |
	.byte $BE ;|X XXXXX |
	.byte $FE ;|XXXXXXX |

DeathFrame2
	.byte $14 ;|   X X  |
	.byte $42 ;| X    X |
	.byte $28 ;|  X X   |
	.byte $1C ;|   XXX  |
	.byte $3E ;|  XXXXX |   
	.byte $FE ;|XXXXXXX |
	.byte $3E ;|  XXXXX |
	.byte $5F ;| X XXXXX|
	.byte $7F ;| XXXXXXX|
	.byte $7F ;| XXXXXXX| mustache
	.byte $3E ;|  XXXXX | chin
	.byte $1C ;|   XXX  | neck
	.byte $3F ;|  XXXXXX|
	.byte $F9 ;|XXXXX  X|
	.byte $3A ;|  XXX X |
	.byte $32 ;|  XX  X |
	.byte $BE ;|X XXXXX |
	.byte $FE ;|XXXXXXX |
	
;
; Coming into the GameKernel
;	y = current group count
;	a = current background color
;
; The background color is used for the playfield background as well as
; the platform color.

StartGameKernel
	ldy groupCount
GameKernel SUBROUTINE
	sta WSYNC
;--------------------------
	sta HMOVE							;3
	sta COLUBK							;3
	lda playerGraphic					;3
	sta GRP0								;3
	lda playerColor					;3
	sta COLUP0							;3
	
	lda PF1PlatformLadderValue,y	;4		end of HBLANK
	sta PF1								;3		@25	needs to be before cycle 27 :)
		
	lda PF2PlatformLadderValue,y	;4
	sta PF2								;3		@32	needs to be before cycle 38 :)
	
	lda #$00								;2
	sta GRP1								;3
	sta playerGraphic					;3
	sta playerBufferedGraph			;3
		
	inx									;2
	cpx #PLAYER_HEIGHT				;2
	bcs PlatformScanline2			;2

	lda playerGraphics,x				;4
	sta playerGraphic					;3
	lda playerColors,x				;4	= 63
	
PlatformScanline2	SUBROUTINE
	sta WSYNC
;--------------------------
	sta HMOVE							;3
	sta COLUP0							;3
	lda playerGraphic					;3
	sta GRP0								;3
	
	lda #$00								;2
	sta playerGraphic					;3
	
	inx									;2
	cpx #PLAYER_HEIGHT				;2
	bcs .bufferPlayerGraphics		;2

	lda playerGraphics,x				;4
	sta playerGraphic					;3
	lda playerColors,x				;4
	sta playerColor					;3
	
.bufferPlayerGraphics
	inx									;2
	cpx #PLAYER_HEIGHT				;2
	bcs .positionGirder				;2

	lda playerGraphics,x				;4
	sta playerBufferedGraph			;3
	lda playerColors,x				;4
	sta playerBufferedColor			;3

.positionGirder
	lda playerGraphic					;3 = 72
	sec
	sta WSYNC
;--------------------------
	sta HMOVE							;3
	sta.w GRP0							;4
	lda playerColor					;3
	sta.w COLUP0						;4
	lda horPosP1-1,y					;4	@ 18
.coarseMoveGirder
	sbc #$0F
	bcs .coarseMoveGirder			;59
	sta RESP1

	sta WSYNC
;--------------------------
	sta HMOVE							;3
	tay									;2
	lda HorizontalMoveTable-240,y	;4
	tay									;2
	
	lda playerBufferedGraph			;3
	sta GRP0								;3
	lda playerBufferedColor			;3
	sta COLUP0							;3
	
	lda #$00								;2
	sta playerGraphic					;3
	sta playerBufferedGraph			;3

	sty HMP1								;3
	
	inx									;2
	cpx #PLAYER_HEIGHT				;2
	bcs .loadGirderGraphics			;2
	lda playerGraphics,x				;2
	sta playerGraphic					;2
	lda playerColors,x				;2
	sta playerColor					;2
.loadGirderGraphics
	
	ldy groupCount						;3
	lda girderGraphics-1,y			;4
	sta GRP1								;3
	
	ldy #GRPHEIGHT						;2
	lda backgroundColor				;3
	
	sta WSYNC
;--------------------------		start of group
	sta HMOVE							;3
	sta COLUBK							;3
	lda playerGraphic					;3
	sta GRP0								;3
	lda playerColor					;3
	sta COLUP0							;3
	lda #$00								;2

	sta PF1								;3
	sta PF2								;3
	
	sta playerGraphic					;3
	sta HMP1								;3
	
	inx									;2
	cpx #PLAYER_HEIGHT				;2
	bcs .decrementY1					;2
	
	lda playerGraphics,x				;4
	sta playerGraphic					;3
	lda playerColors,x				;4
	sta playerColor					;3

.decrementY1	
	dey									;2
	
	sta WSYNC
;--------------------------
	sta HMOVE							;3
	lda playerGraphic					;3
	sta GRP0								;3
	lda playerColor					;3
	sta COLUP0							;3

	lda #$00								;2
	sta playerGraphic					;3
	inx									;2
	cpx #PLAYER_HEIGHT				;2
	bcs .decrementY2					;2
	
	lda playerGraphics,x				;4
	sta playerGraphic					;3
	lda playerColors,x				;4
	sta playerColor					;3
	
.decrementY2
	dey									;2
	
	sty temp								;3
	
	lda groupCount						;3
	cmp #NUMGRP							;2
	bne .setKernelJump				;2
	lda #$02								;2
	bpl .storeIndex					;3
	
.setKernelJump
	and #%00000001						;2
.storeIndex
	tay									;2

	sta WSYNC
;--------------------------
	sta HMOVE							;3
	lda playerColor					;3
	sta COLUP0							;3
	lda playerGraphic					;3
	sta GRP0								;3

	lda KernelLoTbl,y					;4
	sta vecKernel+1					;3
	lda KernelHiTbl,y					;4
	sta vecKernel						;3
	
	ldy temp								;3
	
	lda #$00								;2
	sta playerGraphic					;3
	
	inx									;2
	cpx #PLAYER_HEIGHT				;2
	bcs .decrementY3					;2
	
	lda playerGraphics,x				;4
	sta playerGraphic					;3
	lda playerColors,x				;4
	sta playerColor					;3

.decrementY3
	dey									;2
	
	sta WSYNC
;--------------------------
	sta HMOVE							;3
	lda playerColor					;3
	sta COLUP0							;3
	lda playerGraphic					;3
	sta GRP0								;3

	jsr DrawP0ForEmptyLadderGroup	;6 + 26
	
	dec groupCount						;5
	dey									;2
	jmp (vecKernel)					;5
	
DrawLadderGroup	SUBROUTINE
	sta WSYNC
;--------------------------
	sta HMOVE							;3

	lda playerGraphic					;3
	sta GRP0								;3
	lda playerColor					;3
	sta COLUP0							;3

.setPlayerGraphics
   inx									;2
   cpx #PLAYER_HEIGHT				;2
	bcs .skipGRP0Draw					;2

	lda playerGraphics,x				;4
	sta playerGraphic					;3
	lda playerColors,x				;4
	sta playerColor					;3
	
.nextLadderLine
	lda DivisibleBy4-1,y				;4
	beq .performLadderLoop			;2
	jmp EmptyLadderKernel			;3
.performLadderLoop
	dey									;2
	jmp DrawLadderGroup				;3
	
.skipGRP0Draw
	lda #$00
	sta playerGraphic
	jmp .nextLadderLine
	
DrawOddLadderGroup	

	sta WSYNC
;--------------------------
	sta HMOVE							;3

	lda playerGraphic					;3
	sta GRP0								;3
	lda playerColor					;3
	sta COLUP0							;3
	
   lda #ODD_LADDER_PF1				;2
   sta PF1								;3
   
   lda #ODD_LADDER_PF2				;2
   sta PF2								;3
   
   jmp .setPlayerGraphics	
	
DrawEvenLadderGroup

	sta WSYNC
;--------------------------
	sta HMOVE							;3
	lda playerGraphic					;3

	sta GRP0								;3
	lda playerColor					;3
	sta COLUP0							;3
	lda #EVEN_LADDER_PF1				;2
	sta PF1								;3 @ 20

	lda #EVEN_LADDER_PF2				;2
	sta PF2								;3
	
	jmp .setPlayerGraphics
	
EmptyLadderKernel	SUBROUTINE
	dey									;2
	
	lda groupCount						;3
	cmp #NUMGRP-1						;2
	bcs .enableBall					;2
	lda ladderColor					;3
	bne .drawNextScanline			;3
.enableBall
	lda #NTSC_BALL_COLOR				;2
	bit SWCHB							;4
	bpl .setBallColor					;2
	lda #PAL_BALL_COLOR				;2
.setBallColor
	cpy #$07								;2
	bcc .drawNextScanline			;2
	lda ladderColor					;3
	
.drawNextScanline
	sta WSYNC
;--------------------------
	sta HMOVE							;3
	sta ENABL							;3
	sta COLUPF							;3
	lda playerGraphic					;3
	sta GRP0								;3
	lda playerColor					;3
	sta COLUP0							;3
	lda #$00								;2
	sta PF1								;3
	sta PF2								;3
	sta playerBufferedGraph			;3
	
	jsr DrawP0ForEmptyLadderGroup	;6 + 26
	
	dey									;2
	
	sta WSYNC
;--------------------------
	sta HMOVE							;3
	lda playerGraphic					;3
	sta GRP0								;3
	lda playerColor					;3
	sta COLUP0							;3
	
	jsr DrawP0ForEmptyLadderGroup	;6 + 26

	dey									;2

	sta WSYNC
;--------------------------
	sta HMOVE							;3
	lda playerGraphic					;3
	sta GRP0								;3
	lda playerColor					;3
	sta COLUP0							;3

	inx									;2
	cpx #PLAYER_HEIGHT				;2
	bcs .setP0Graphics				;2

	lda playerGraphics,x				;4
	sta playerBufferedGraph			;3
	lda playerColors,x				;4
	sta playerBufferedColor			;3
	
.setP0Graphics
	jsr DrawP0ForEmptyLadderGroup	;6 + 26
	dey									;2

LastEmptyLadderKernel	SUBROUTINE	
	sta WSYNC
;--------------------------
	sta HMOVE							;3
	
	lda playerBufferedGraph			;3
	sta GRP0								;3
	lda playerBufferedColor			;3
	sta COLUP0							;3

	dey									;2
	beq .continueGameKernel			;2
	
	jmp (vecKernel)					;5

.continueGameKernel
	sty GRP1								;-2
	lda ladderColor					;3
	sta COLUPF							;3	roughly 75% of the viewable screen
	lda #$00								;2
	sta ENABL							;3
	lda platformColor					;3	position is <= ~ 120
	ldy groupCount						;3
	beq ScoreKernel
	jmp GameKernel						;3


DrawP0ForEmptyLadderGroup	SUBROUTINE
   inx									;2
   cpx #PLAYER_HEIGHT				;2
	bcs .skipP0Draw					;2
	
	lda playerGraphics,x				;4
	sta playerGraphic					;3
	lda playerColors,x				;4
	sta playerColor					;3
	rts									;6		=	26
	
.skipP0Draw
	lda #$00								;2
	sta playerGraphic					;3
	rts									;6		=	18

;
; The game screen has been drawn. Now the score, player indicators, and
; level number must be drawn.
;
; The game screen takes up 168 scan lines
;		(4 lines for platform + 20 lines for group) * 7 groups)
; For NTSC (192 scan lines) there are 24 scan lines left.
;
ScoreKernel	SUBROUTINE
	lda platformColor
	ldy #$00
	sta WSYNC
;--------------------------
	sta HMOVE							;3
	sta COLUBK							;3
	sty PF1								;3
	sty GRP0								;3
	sty GRP1								;3
	sty ENABL							;3
	sty PF2								;3 @21
	
	ldx #$03								;2
	stx NUSIZ0							; 3 copies of GRP0 close
	stx NUSIZ1							; 3 copies of GRP1 close
	stx VDELP0							; 3 @ 32

	lda #>Level0				;2	
	sta levelPointer+1		;3
	
	sta RESP0					;3
	sta RESP1					;3
	
	sta levelPointer+3		;3
	sta levelPointer+5		;3
	sta levelPointer+7		;3
	
	lda #HMOVE_L1				;2
	sta HMP0						;3
	
	lda #HMOVE_L2				;2
	sta HMP1						;3

	lda #>Blank					;2
	sta levelPointer+9		;3
	sta levelPointer+11		;3
	
	lda #<Blank					;2 @ 72
	sta.w levelPointer+8		;4
;-------------------------
	sta HMOVE					;3
	sta levelPointer+10		;3
	
	lda #<Level0				;2
	sta levelPointer			;3
	
	lda #<Level1				;2
	sta levelPointer+2		;3
	
	lda #<Level2				;2
	sta levelPointer+4		;3
	
	lda #<Level3				;2
	sta levelPointer+6		;3	@26

	lda level					;3
   and #$0F						;2
   sta temp						;3
   asl                     ;2
   asl                     ;2
	clc							;2
	adc temp						;3
   sta graphicsPointers+10	;3 @46
 
   lda #BLACK					;2
   sta COLUP0					;3  colour
   sta COLUP1					;3

   sta REFP0					;3
   sta REFP1					;3
   
   lda #GAME_TEXT_HEIGHT-1	;2
   sta.w fontHeight			;3
   sta HMCLR					;3 @69
   
   lda level					;3
   and #$F0						;2
   lsr
;-------------------------
	sta HMOVE
	beq .drawLevelInformation
	adc #<NumberFonts
	sta levelPointer+8
.drawLevelInformation
   jsr DrawIt					;9
	
	ldx lives					;3
	bmi .drawGameOver
	dex							;2

	bpl .displayReservedLives;2
	ldx #$17
	bne .loop

.drawGameOver;@15
	ldx #$0B
.storeGameOverLoop
	ldy #$03
	sta WSYNC
;-------------------------
	sta HMOVE
.gameOverGraphicLoop
	lda GameOverLiteral,x		;4
	sta levelPointer,x			;4
	dex								;2
	dey
	bpl .gameOverGraphicLoop
	txa
	bpl .storeGameOverLoop
	
	sta WSYNC
;-------------------------
	sta HMOVE
	lda #GAME_TEXT_HEIGHT-1	;2
	sta fontHeight				;3
	
	jsr DrawIt
	
	ldx #$0D
	bne .loop

.displayReservedLives;@19
	sta RESP0
	sty VDELP0
	stx NUSIZ0
	lda %00001000
	sta REFP0
	
	ldx #RESERVE_LIVES_HEIGHT
	ldy #$00
.drawReservedLives
	sta WSYNC
;-------------------------
	sta HMOVE
	lda HorizontalMoveFrame1,y
	sta GRP0
	iny
	dex
	bpl .drawReservedLives
	inx
	stx GRP0
	
	ldx #$0E
.loop
	sta WSYNC
	sta HMOVE
	dex
	bpl .loop
	jmp Overscan
	
;----------------------------------------------------------------RestartLevel
;
RestartLevel SUBROUTINE
	ldx #NUMGRP-1
	ldy #$01
.resetStartCountLoop
	jsr RandomByte
	and #$3F
	sta startCount,x
	lda #XMIN
	sta horPosP1,x
	dex
	bmi .setGirderGraphics
	sty startCount,x
	lda #XMIN
	sta horPosP1,x
	dex
	bpl .resetStartCountLoop
	
.setGirderGraphics
	inx
	stx gameClock
	stx playerState
	stx frameCount
	stx REFP0
	
	bit SWCHB								; check for PAL or NTSC setting
	bpl .setPlayerColors
	ldx #$02
.setPlayerColors
	lda HorizontalColorTable,x
	sta playerColorPointer
	lda HorizontalColorTable+1,x
	sta playerColorPointer+1
	
.setClimberStartPosition
	lda #CLIMBER_X_START
	sta horPosP0
	lda #CLIMBER_Y_START
	sta verPosP0
	
	lda #>HorizontalMoveFrame1
	sta playerPointer+1
	
	lda #<HorizontalMoveFrame1
	sta playerPointer
	rts
	
NTSCFontColorTable
	.byte #NTSC_WHITE
	.byte #NTSC_BROWN
	.byte #NTSC_BROWN
	.byte #NTSC_WHITE
	.byte #NTSC_ATARI_AGE_BLUE
	
PALFontColorTable
	.byte #PAL_WHITE
	.byte #PAL_BROWN
	.byte #PAL_BROWN
	.byte #PAL_WHITE
	.byte #PAL_ATARI_AGE_BLUE
	
;-----------------------------------------------------------SetGirderGraphics
;
SetGirderGraphics SUBROUTINE
	ldy #GIRDER_GRAPHIC
	bit gameSelection
	bpl .normalGameSetGirderGraphics
	
	lda level
	cmp #STARTING_LEVEL
	bne .doneGirderGraphics
	
	jsr RandomByte
.setRandomGirderGraphicLoop
	cmp #NUMGRP
	bcc .determineRandomGraphic
	lda #NUMGRP-1
.determineRandomGraphic
	tax
	lda girderGraphics,x
	beq .storeGirderGraphics
	dex
	txa
	jmp .setRandomGirderGraphicLoop
	
.storeGirderGraphics	
	sty girderGraphics,x
	rts
	
.normalGameSetGirderGraphics
	ldx #NUMGRP-1
.setGirderGraphicsLoop
	sty girderGraphics,x
	dex
	bpl .setGirderGraphicsLoop
.doneGirderGraphics
	rts
	
;-------------------------------------------------------------------PlaySound
;
; The volume and channel will be set this routine. The x register must hold
; the frequency. This sound routine only manipulates sound channel 1
;
PlaySound SUBROUTINE
	lda #GAME_SOUND_VOLUME
	sta AUDV0
	lda #$0C
	sta AUDC0
	stx AUDF0
	dec soundIndex
	rts
	
;----------------------------------------------------------TitleDisplayKernel
;
; The title screen display is drawn here.
;
TitleDisplayKernel SUBROUTINE
	lda #$04
	sta groupCount
	sta WSYNC
;--------------------------
	sta HMOVE

.atariAgePresentsKernel
	lda #$02
	sta loopCount
	jsr DisplayGraphicsLiteral
	
.climber5Kernel
	sta WSYNC
;--------------------------
	sta HMOVE
	
	sta WSYNC
;--------------------------
	sta HMOVE
	
	sta WSYNC
;--------------------------
	sta HMOVE
	
	ldx #$02
	stx loopCount
	
	sta WSYNC
;--------------------------
	sta HMOVE
	lda #%11111110
	sta PF2
	
	sta WSYNC
;--------------------------
	sta HMOVE
	
	sta WSYNC
;--------------------------
	sta HMOVE
	
	jsr DisplayGraphicsLiteral
	
	sta WSYNC
;--------------------------
	sta HMOVE
	
	sta WSYNC
;--------------------------
	sta HMOVE
	
	lda #$00
	sta PF2
	
	ldx #$28
.skipLinesLoop
	sta WSYNC
;-------------------------
	sta HMOVE
	dex
	bpl .skipLinesLoop

	lda #<TrainingLiteral
	sta graphicGroupPointer
	
	lda #>TrainingLiteral
	sta graphicGroupPointer+1
	
	lda #TEXT_HEIGHT-1
	sta fontHeight
	
	lda #NTSC_WHITE
	sta COLUP0
	sta COLUP1
	
	lda gameSelection
	bpl .trainingModeFontColorSet
	
	lda #NTSC_RED
	bit SWCHB
	bpl .setTrainingModeFontColor
	lda #PAL_RED
.setTrainingModeFontColor
	sta COLUP0
	sta COLUP1
.trainingModeFontColorSet
	lda #$01
	sta loopCount
	jsr BeginningGraphicLoop
	
	sta WSYNC
;-------------------------
	sta HMOVE
	
	lda #<NormalLiteral
	sta graphicGroupPointer
	
	lda #>NormalLiteral
	sta graphicGroupPointer+1
	
	lda #TEXT_HEIGHT-1
	sta fontHeight
	
	lda #NTSC_WHITE
	sta COLUP0
	sta COLUP1
	
	lda gameSelection
	bmi .normalModeFontColorSet
	
	lda #NTSC_RED
	bit SWCHB
	bpl .setNormalModeFontColor
	lda #PAL_RED
.setNormalModeFontColor
	sta COLUP0
	sta COLUP1
.normalModeFontColorSet
	lda #$01
	sta loopCount
	jsr BeginningGraphicLoop
	
;	bit SWCHB
;	bpl .loop
	ldx #$17;#$14
.loop
	sta WSYNC
;-------------------------
	sta HMOVE
	dex
	bpl .loop
	
	inx
	stx groupCount
	inx
	stx loopCount
	jsr DisplayGraphicsLiteral
	
	jmp Overscan
		
	
DisplayGraphicsLiteral SUBROUTINE
	sta WSYNC
;-------------------------
	sta HMOVE							;3
	
	ldy groupCount						;2
	lda GraphicsGroupLow,y			;4
	sta graphicGroupPointer			;3
	lda GraphicsGroupHigh,y			;4
	sta graphicGroupPointer+1		;3
	lda FontHeightTable,y			;3
	sta fontHeight						;3
	
	ldx #$00								;2
	bit SWCHB							;4
	bpl .loadFontColors				;2
	ldx #$01								;2
.loadFontColors
	lda FontColorPointerLo,x		;4
	sta fontColorPointer				;3
	
	lda FontColorPointerHi,x		;4
	sta fontColorPointer+1			;3
	
	lda (fontColorPointer),y		;5
	
.setFontColors
	sta COLUP0							;3
	sta COLUP1							;3
	
BeginningGraphicLoop
	ldy #$0B
	ldx #$02

.storeGraphicLoop	
	sta WSYNC
;-------------------------
	sta HMOVE							;3
.graphicPointerLoop
	lda (graphicGroupPointer),y	;5
	sta graphicsPointers,y			;5
	dey									;2
	dex									;2
	bpl .graphicPointerLoop			;2

	ldx #$02
	tya
	bpl .storeGraphicLoop
	
	sta WSYNC
;-------------------------
	sta HMOVE
	
	jsr DrawIt
	dec groupCount
	dec loopCount
	bne DisplayGraphicsLiteral
	rts
	
FontColorPointerHi
	.byte #>NTSCFontColorTable
	.byte #>PALFontColorTable

FontColorPointerLo
	.byte #<NTSCFontColorTable
	.byte #<PALFontColorTable
	
GraphicsGroupLow
	.byte #<CopyrightLiteral
	.byte #<Climber5Literal
	.byte #<ClimberLiteral
	.byte #<PresentsLiteral
	.byte #<AtariAgeLiteral
	
GraphicsGroupHigh
	.byte #>CopyrightLiteral
	.byte #>Climber5Literal
	.byte #>ClimberLiteral
	.byte #>PresentsLiteral
	.byte #>AtariAgeLiteral
	
AtariAgeLiteral
	.word AtariAgeSymbol0
	.word AtariAgeSymbol1
	.word AtariAge0
	.word AtariAge1
	.word AtariAge2
	.word AtariAge3
	
PresentsLiteral
	.word Blank
	.word Presents0
	.word Presents1
	.word Presents2
	.word Presents3
	.word Blank
	
ClimberLiteral
	.word Climber0
	.word Climber1
	.word Climber2
	.word Climber3
	.word Climber4
	.word Climber5	
	
Climber5Literal
	.word Blank
	.word Climber50
	.word Climber51
	.word Climber52
	.word Climber53
	.word Blank
	
NormalLiteral
	.word Blank
	.word Normal0
	.word Normal1
	.word Normal2
	.word Normal3
	.word Blank
	
TrainingLiteral
	.word Blank
	.word Training0
	.word Training1
	.word Training2
	.word Training3
	.word Blank
	
CopyrightLiteral
	.word Copyright0
	.word Copyright1
	.word Copyright2
	.word Copyright3
	.word Copyright4
	.word Copyright5
	
GameOverLiteral
	.word GameOver0
	.word GameOver1
	.word GameOver2
	.word GameOver3
	.word GameOver4
	.word GameOver5
	
FontHeightTable
	.byte #COPYRIGHT_HEIGHT-1
	.byte #NUMBER_5_HEIGHT-1
	.byte #CLIMBER_FONT_HEIGHT-1
	.byte #PRESENTS_FONT_HEIGHT-1
	.byte #AA_FONT_HEIGHT-1
	


AtariAgeSymbol0
	.byte %00000111;|     XXX|
	.byte %00001000;|    X   |
	.byte %00010111;|   X XXX|
	.byte %00101111;|  X XXXX|
	.byte %01011111;| X XXXXX|
	.byte %01010111;| X X XXX|
	.byte %01011011;| X XX XX|
	.byte %01011101;| X XXX X|
	.byte %01010110;| X X XX |
	.byte %01011011;| X XX XX|
	.byte %01011101;| X XXX X|
	.byte %00101110;|  X XXX |
	.byte %00010111;|   X XXX|
	.byte %00001000;|    X   |
	.byte %00000111;|     XXX|			15
	
AtariAgeSymbol1
	.byte %11000000;|XX      |
	.byte %00100000;|  X     |
	.byte %11010000;|XX X    |
	.byte %11101000;|XXX X   |
	.byte %11110100;|XXXX X  |
	.byte %11010100;|XX X X  |
	.byte %10110100;|X XX X  |
	.byte %01110100;| XXX X  |
	.byte %11010100;|XX X X  |
	.byte %10110100;|X XX X  |
	.byte %01110100;| XXX X  |
	.byte %11101000;|XXX X   |
	.byte %11010000;|XX X    |
	.byte %00100000;|  X     |
	.byte %11000000;|XX      |			15

AtariAge0
	.byte %00000000;|        |
	.byte %00000000;|        |
	.byte %00000000;|        |
	.byte %00000000;|        |
	.byte %00000000;|        |
	.byte %10100010;|X X   X |
	.byte %10100100;|X X  X  |
	.byte %10100100;|X X  X  |
	.byte %11100100;|XXX  X  |
	.byte %10101110;|X X XXX |
	.byte %10100100;|X X  X  |
	.byte %01000000;| X      |
	.byte %00000000;|        |
	.byte %00000000;|        |
	.byte %00000000;|        |			15

AtariAge1
	.byte %00000000;|        |
	.byte %00000000;|        |
	.byte %00000000;|        |
	.byte %00000000;|        |
	.byte %00000000;|        |
	.byte %01110100;| XXX X  |
	.byte %10010100;|X  X X  |
	.byte %01110100;| XXX X  |
	.byte %00010110;|   X XX |
	.byte %01100101;| XX  X X|
	.byte %00000000;|        |
	.byte %00000000;|        |
	.byte %00000000;|        |
	.byte %00000000;|        |
	.byte %00000000;|        |			15

AtariAge2
	.byte %00000000;|        |
	.byte %00000000;|        |
	.byte %00000000;|        |
	.byte %00000000;|        |
	.byte %00000000;|        |
	.byte %00101010;|  X X X |
	.byte %00101010;|  X X X |
	.byte %00101010;|  X X X |
	.byte %10101110;|X X XXX |
	.byte %00101010;|  X X X |
	.byte %00001010;|    X X |
	.byte %00100100;|  X  X  |
	.byte %00000000;|        |
	.byte %00000000;|        |
	.byte %00000000;|        |			15
	
AtariAge3
	.byte %00000000;|        |
	.byte %00000000;|        |
	.byte %00000000;|        |
	.byte %01100000;| XX     |
	.byte %10010000;|X  X    |
	.byte %00010011;|   X  XX|
	.byte %01110100;| XXX X  |
	.byte %10010111;|X  X XXX|
	.byte %01110101;| XXX X X|
	.byte %00000010;|      X |
	.byte %00000000;|        |
	.byte %00000000;|        |
	.byte %00000000;|        |
	.byte %00000000;|        |
	.byte %00000000;|        |			15
	
Presents0
	.byte %10000000;|X       |
	.byte %10000000;|X       |
	.byte %11101000;|XXX X   |
	.byte %10101000;|X X X   |
	.byte %10101000;|X X X   |
	.byte %10101000;|X X X   |
	.byte %11101110;|XXX XXX |
	.byte %00000000;|        |			8
	
Presents1
	.byte %00000000;|        |
	.byte %00000000;|        |
	.byte %11101110;|XXX XXX |
	.byte %10000010;|X     X |
	.byte %11101110;|XXX XXX |
	.byte %10101000;|X X X   |
	.byte %11101110;|XXX XXX |
	.byte %00000000;|        |			8
	
Presents2
	.byte %00000000;|        |
	.byte %00000000;|        |
	.byte %11101010;|XXX X X |
	.byte %10001010;|X   X X |
	.byte %11101010;|XXX X X |
	.byte %10101010;|X X X X |
	.byte %11101110;|XXX XXX |
	.byte %00000000;|        |			8
	
Presents3
	.byte %00000000;|        |
	.byte %00000000;|        |
	.byte %01101110;| XX XXX |
	.byte %01000010;| X    X |
	.byte %01001110;| X  XXX |
	.byte %01001000;| X  X   |
	.byte %11101110;|XXX XXX |
	.byte %01000000;| X      |			8

Climber50
	.byte %00000000;|        |
	.byte %00000001;|       X|
	.byte %00000000;|        |
	.byte %00000001;|       X|
	.byte %00000001;|       X|
	.byte %00000001;|       X|			6
	
Climber51
	.byte %00111111;|  XXXXXX|
	.byte %11111000;|XXXXX   |
	.byte %00000000;|        |
	.byte %11111111;|XXXXXXXX|
	.byte %11111000;|XXXXX   |
	.byte %11111111;|XXXXXXXX|			6
	
Climber52
	.byte %11111100;|XXXXXX  |
	.byte %00011111;|   XXXXX|
	.byte %00011111;|   XXXXX|
	.byte %11111100;|XXXXXX  |
	.byte %00000000;|        |
	.byte %11111111;|XXXXXXXX|			6

Climber53
	.byte %00000000;|        |
	.byte %10000000;|X       |
	.byte %10000000;|X       |
	.byte %00000000;|        |
	.byte %00000000;|        |
	.byte %10000000;|X       |			6	
	
Normal0
	.byte %01001001;| X  X  X|
	.byte %01001010;| X  X X |
	.byte %01011010;| X XX X |
	.byte %01101010;| XX X X |
	.byte %01001001;| X  X  X|
	
Normal1
	.byte %10010010;|X  X  X |
	.byte %01010010;| X X  X |
	.byte %01011100;| X XXX  |
	.byte %01010010;| X X  X |
	.byte %10011100;|X  XXX  |
	
Normal2
	.byte %10001010;|X   X X |
	.byte %10101010;|X X X X |
	.byte %10101011;|X X X XX|
	.byte %11011010;|XX XX X |
	.byte %10001001;|X   X  X|
	
Normal3
	.byte %01011110;| X XXXX |
	.byte %01010000;| X X    |
	.byte %11010000;|XX X    |
	.byte %01010000;| X X    |
	.byte %10010000;|X  X    |
	
HorizontalColorTable
	.word NTSCHorizontalColors
	.word PALHorizontalColors
	
VerticalColorTable
	.word NTSCVerticalColors
	.word PALVerticalColors

HighVerticalMovementTable
	.byte #>VerticalMoveFrame1
	.byte #>VerticalMoveFrame2
	
LowVerticalMovementTable
	.byte #<VerticalMoveFrame1
	.byte #<VerticalMoveFrame2

HighDeathAnimationTable
	.byte #>DeathFrame1
	.byte #>DeathFrame2
	
LowDeathAnimationTable
	.byte #<DeathFrame1
	.byte #<DeathFrame2
	
Copyright0	
	.byte %00111000;|  XXX   |
	.byte %01000100;| X   X  |
	.byte %10111010;|X XXX X |
	.byte %10100010;|X X   X |
	.byte %10111010;|X XXX X |
	.byte %01000100;| X   X  |
	.byte %00111000;|  XXX   |
	.byte %00000000;|        |

Copyright1
	.byte %00000000;|        |
	.byte %11101110;|XXX XXX |
	.byte %10001010;|X   X X |
	.byte %11101010;|XXX X X |
	.byte %00101010;|  X X X |
	.byte %11101110;|XXX XXX |
	.byte %00000000;|        |
	.byte %00000000;|        |

Copyright2
	.byte %00000000;|        |
	.byte %11101110;|XXX XXX |
	.byte %10100010;|X X   X |
	.byte %10101110;|X X XXX |
	.byte %10100010;|X X   X |
	.byte %11101110;|XXX XXX |
	.byte %00000000;|        |
	.byte %00000000;|        |

Copyright3
	.byte %00000000;|        |
	.byte %11010110;|XX X XX |
	.byte %10100101;|X X  X X|
	.byte %10100101;|X X  X X|
	.byte %10100101;|X X  X X|
	.byte %10100101;|X X  X X|
	.byte %10100101;|X X  X X|
	.byte %11000110;|XX   XX |

Copyright4
	.byte %00000000;|        |
	.byte %01110111;| XXX XXX|
	.byte %01000101;| X   X X|
	.byte %01110101;| XXX X X|
	.byte %01010101;| X X X X|
	.byte %01110111;| XXX XXX|
	.byte %00000100;|     X  |
	.byte %00000100;|     X  |

Copyright5
	.byte %00000000;|        |
	.byte %01000111;| X   XXX|
	.byte %01000101;| X   X X|
	.byte %01000101;| X   X X|
	.byte %01000101;| X   X X|
	.byte %01110111;| XXX XXX|
	.byte %00000000;|        |
	.byte %00000000;|        |

Training0
	.byte %01001001;| X  X  X|
	.byte %01001001;| X  X  X|
	.byte %01001110;| X  XXX |
	.byte %01001001;| X  X  X|
	.byte %11101110;|XXX XXX |
	
Training1
	.byte %01001010;| X  X X |
	.byte %01001010;| X  X X |
	.byte %01111010;| XXXX X |
	.byte %01001010;| X  X X |
	.byte %00110010;|  XX  X |
	
Training2
	.byte %10010101;|X  X X X|
	.byte %10010101;|X  X X X|
	.byte %10110101;|X XX X X|
	.byte %11010101;|XX X X X|
	.byte %10010101;|X  X X X|
	
Training3
	.byte %00100111;|  X  XXX|
	.byte %00101001;|  X X  X|
	.byte %01101011;| XX X XX|
	.byte %10101000;|X X X   |
	.byte %00100111;|  X  XXX|


	
HighHorizontalMovementTable
	.byte #>HorizontalMoveFrame1
	.byte #>HorizontalMoveFrame2
	.byte #>HorizontalMoveFrame3
	.byte #>HorizontalMoveFrame1
	
LowHorizontalMovementTable
	.byte #<HorizontalMoveFrame1
	.byte #<HorizontalMoveFrame2
	.byte #<HorizontalMoveFrame3
	.byte #<HorizontalMoveFrame1
	
NTSCHorizontalColors	
	.byte #NTSC_GREEN
	.byte #NTSC_GREEN
	.byte #NTSC_GREEN
	.byte #NTSC_PEACH
	.byte #NTSC_PEACH
	.byte #NTSC_PEACH
	.byte #NTSC_PEACH
	.byte #NTSC_PEACH
	.byte #NTSC_PEACH
	.byte #NTSC_GREEN
	.byte #NTSC_GREEN
	.byte #NTSC_GREEN
	.byte #NTSC_GREEN
	.byte #NTSC_GREEN
	.byte #NTSC_GREEN
	.byte #NTSC_GREEN
	.byte #NTSC_GREEN	
	.byte #NTSC_LTBLUE
	
PALHorizontalColors	
	.byte #PAL_GREEN
	.byte #PAL_GREEN
	.byte #PAL_GREEN
	.byte #PAL_PEACH
	.byte #PAL_PEACH
	.byte #PAL_PEACH
	.byte #PAL_PEACH
	.byte #PAL_PEACH
	.byte #PAL_PEACH
	.byte #PAL_GREEN
	.byte #PAL_GREEN
	.byte #PAL_GREEN
	.byte #PAL_GREEN
	.byte #PAL_GREEN
	.byte #PAL_GREEN
	.byte #PAL_GREEN
	.byte #PAL_GREEN	
	.byte #PAL_LTBLUE

NTSCVerticalColors   
	.byte #NTSC_GREEN
	.byte #NTSC_GREEN
   .byte #NTSC_GREEN
   .byte #NTSC_GREEN
   .byte #NTSC_PEACH
   .byte #NTSC_PEACH
   .byte #NTSC_GREEN
   .byte #NTSC_GREEN
   .byte #NTSC_GREEN
   .byte #NTSC_GREEN
   .byte #NTSC_GREEN
   .byte #NTSC_GREEN
   .byte #NTSC_GREEN
   .byte #NTSC_GREEN
   .byte #NTSC_GREEN
   .byte #NTSC_GREEN
   .byte #NTSC_GREEN   
   .byte #NTSC_LTBLUE
   
PALVerticalColors   
	.byte #PAL_GREEN
	.byte #PAL_GREEN
   .byte #PAL_GREEN
   .byte #PAL_GREEN
   .byte #PAL_PEACH
   .byte #PAL_PEACH
   .byte #PAL_GREEN
   .byte #PAL_GREEN
   .byte #PAL_GREEN
   .byte #PAL_GREEN
   .byte #PAL_GREEN
   .byte #PAL_GREEN
   .byte #PAL_GREEN
   .byte #PAL_GREEN
   .byte #PAL_GREEN
   .byte #PAL_GREEN
   .byte #PAL_GREEN   
   .byte #PAL_LTBLUE
	
NTSCDeathAnimationColors
	.byte #NTSC_WHITE
	.byte #NTSC_WHITE
	.byte #NTSC_WHITE
	.byte #NTSC_GREEN
	.byte #NTSC_GREEN
	.byte #NTSC_GREEN
	.byte #NTSC_PEACH
	.byte #NTSC_PEACH
	.byte #NTSC_PEACH
	.byte #NTSC_PEACH
	.byte #NTSC_PEACH
	.byte #NTSC_PEACH
	.byte #NTSC_GREEN
	.byte #NTSC_GREEN
	.byte #NTSC_GREEN
	.byte #NTSC_GREEN
	.byte #NTSC_GREEN
	.byte #NTSC_GREEN
	
PALDeathAnimationColors
	.byte #PAL_WHITE
	.byte #PAL_WHITE
	.byte #PAL_WHITE
	.byte #PAL_GREEN
	.byte #PAL_GREEN
	.byte #PAL_GREEN
	.byte #PAL_PEACH
	.byte #PAL_PEACH
	.byte #PAL_PEACH
	.byte #PAL_PEACH
	.byte #PAL_PEACH
	.byte #PAL_PEACH
	.byte #PAL_GREEN
	.byte #PAL_GREEN
	.byte #PAL_GREEN
	.byte #PAL_GREEN
	.byte #PAL_GREEN
	.byte #PAL_GREEN
		
DeathAnimationColorTable
	.word NTSCDeathAnimationColors
	.word PALDeathAnimationColors
	
	align 256
	
Climber0
	.byte %00111100;|  XXXX  |
	.byte %00111100;|  XXXX  |
	.byte %00111100;|  XXXX  |
	.byte %00111100;|  XXXX  |
	.byte %01100110;| XX  XX |
	.byte %01100110;| XX  XX |
	.byte %01100110;| XX  XX |
	.byte %01100110;| XX  XX |
	.byte %01100000;| XX     |
	.byte %01100000;| XX     |
	.byte %01100000;| XX     |
	.byte %01100000;| XX     |
	.byte %01100000;| XX     |
	.byte %01100000;| XX     |
	.byte %01100000;| XX     |
	.byte %01100000;| XX     |
	.byte %01100000;| XX     |
	.byte %01100000;| XX     |
	.byte %01100000;| XX     |
	.byte %01100000;| XX     |
	.byte %01100000;| XX     |
	.byte %01100000;| XX     |
	.byte %01100000;| XX     |
	.byte %01100000;| XX     |
	.byte %01100110;| XX  XX |
	.byte %01100110;| XX  XX |
	.byte %01100110;| XX  XX |
	.byte %01100110;| XX  XX |
	.byte %00111100;|  XXXX  |
	.byte %00111100;|  XXXX  |
	.byte %00111100;|  XXXX  |
	.byte %00111100;|  XXXX  |			32
	
Climber1
	.byte %11111101;|XXXXXX X|
	.byte %11111101;|XXXXXX X|
	.byte %11111101;|XXXXXX X|
	.byte %11111101;|XXXXXX X|
	.byte %11000001;|XX     X|
	.byte %11000001;|XX     X|
	.byte %11000001;|XX     X|
	.byte %11000001;|XX     X|
	.byte %11000001;|XX     X|
	.byte %11000001;|XX     X|
	.byte %11000001;|XX     X|
	.byte %11000001;|XX     X|
	.byte %11000001;|XX     X|
	.byte %11000001;|XX     X|
	.byte %11000001;|XX     X|
	.byte %11000001;|XX     X|
	.byte %11000001;|XX     X|
	.byte %11000001;|XX     X|			8
	.byte %11000001;|XX     X|
	.byte %11000001;|XX     X|
	.byte %11000001;|XX     X|
	.byte %11000001;|XX     X|
	.byte %11000001;|XX     X|
	.byte %11000001;|XX     X|
	.byte %11000001;|XX     X|			8
	.byte %11000001;|XX     X|
	.byte %11000001;|XX     X|
	.byte %11000001;|XX     X|
	.byte %11000001;|XX     X|
	.byte %11000001;|XX     X|
	.byte %11000001;|XX     X|
	.byte %11000001;|XX     X|

Climber2
	.byte %10110000;|X XX    |
	.byte %10110000;|X XX    |
	.byte %10110000;|X XX    |
	.byte %10110000;|X XX    |
	.byte %10110000;|X XX    |
	.byte %10110000;|X XX    |
	.byte %10110000;|X XX    |
	.byte %10110000;|X XX    |
	.byte %10110110;|X XX XX |
	.byte %10110110;|X XX XX |
	.byte %10110110;|X XX XX |
	.byte %10110110;|X XX XX |
	.byte %10110110;|X XX XX |
	.byte %10110110;|X XX XX |
	.byte %10110110;|X XX XX |
	.byte %10110110;|X XX XX |
	.byte %10110110;|X XX XX |
	.byte %10110110;|X XX XX |
	.byte %10110110;|X XX XX |
	.byte %10110110;|X XX XX |
	.byte %10111111;|X XXXXXX|
	.byte %10111111;|X XXXXXX|
	.byte %10111111;|X XXXXXX|
	.byte %10111111;|X XXXXXX|
	.byte %10111001;|X XXX  X|
	.byte %10111001;|X XXX  X|
	.byte %10111001;|X XXX  X|
	.byte %10111001;|X XXX  X|
	.byte %10110000;|X XX    |			8
	.byte %10110000;|X XX    |			8
	.byte %10110000;|X XX    |			8
	.byte %10110000;|X XX    |			8

Climber3
	.byte %11011111;|XX XXXXX|
	.byte %11011111;|XX XXXXX|
	.byte %11011111;|XX XXXXX|
	.byte %11011111;|XX XXXXX|
	.byte %11011001;|XX XX  X|
	.byte %11011001;|XX XX  X|
	.byte %11011001;|XX XX  X|
	.byte %11011001;|XX XX  X|
	.byte %11011001;|XX XX  X|
	.byte %11011001;|XX XX  X|
	.byte %11011001;|XX XX  X|
	.byte %11011001;|XX XX  X|
	.byte %11011001;|XX XX  X|
	.byte %11011001;|XX XX  X|
	.byte %11011001;|XX XX  X|
	.byte %11011001;|XX XX  X|
	.byte %11011110;|XX XXXX |
	.byte %11011110;|XX XXXX |
	.byte %11011110;|XX XXXX |
	.byte %11011110;|XX XXXX |
	.byte %11011001;|XX XX  X|
	.byte %11011001;|XX XX  X|
	.byte %11011001;|XX XX  X|
	.byte %11011001;|XX XX  X|
	.byte %11011001;|XX XX  X|
	.byte %11011001;|XX XX  X|
	.byte %11011001;|XX XX  X|
	.byte %11011001;|XX XX  X|
	.byte %11011111;|XX XXXXX|			8
	.byte %11011111;|XX XXXXX|			8
	.byte %11011111;|XX XXXXX|			8
	.byte %11011111;|XX XXXXX|			8

Climber4
	.byte %00111111;|  XXXXXX|
	.byte %00111111;|  XXXXXX|
	.byte %00111111;|  XXXXXX|
	.byte %00111111;|  XXXXXX|
	.byte %10110000;|X XX    |
	.byte %10110000;|X XX    |
	.byte %10110000;|X XX    |
	.byte %10110000;|X XX    |
	.byte %10110000;|X XX    |
	.byte %10110000;|X XX    |
	.byte %10110000;|X XX    |
	.byte %10110000;|X XX    |
	.byte %10110000;|X XX    |
	.byte %10110000;|X XX    |
	.byte %10110000;|X XX    |
	.byte %10110000;|X XX    |
	.byte %00111110;|  XXXXX |
	.byte %00111110;|  XXXXX |
	.byte %00111110;|  XXXXX |
	.byte %00111110;|  XXXXX |
	.byte %10110000;|X XX    |
	.byte %10110000;|X XX    |
	.byte %10110000;|X XX    |
	.byte %10110000;|X XX    |
	.byte %10110000;|X XX    |
	.byte %10110000;|X XX    |
	.byte %10110000;|X XX    |
	.byte %10110000;|X XX    |
	.byte %00111111;|  XXXXXX|			8
	.byte %00111111;|  XXXXXX|			8
	.byte %00111111;|  XXXXXX|			8
	.byte %00111111;|  XXXXXX|			8
	
Climber5
	.byte %01100110;| XX  XX |
	.byte %01100110;| XX  XX |
	.byte %01100110;| XX  XX |
	.byte %01100110;| XX  XX |
	.byte %01100110;| XX  XX |
	.byte %01100110;| XX  XX |
	.byte %01100110;| XX  XX |
	.byte %01100110;| XX  XX |
	.byte %01100110;| XX  XX |
	.byte %01100110;| XX  XX |
	.byte %01100110;| XX  XX |
	.byte %01100110;| XX  XX |
	.byte %01100110;| XX  XX |
	.byte %01100110;| XX  XX |
	.byte %01100110;| XX  XX |
	.byte %01100110;| XX  XX |
	.byte %01111000;| XXXX   |
	.byte %01111000;| XXXX   |
	.byte %01111000;| XXXX   |
	.byte %01111000;| XXXX   |
	.byte %01100110;| XX  XX |
	.byte %01100110;| XX  XX |
	.byte %01100110;| XX  XX |
	.byte %01100110;| XX  XX |
	.byte %01100110;| XX  XX |
	.byte %01100110;| XX  XX |
	.byte %01100110;| XX  XX |
	.byte %01100110;| XX  XX |
	.byte %01111100;| XXXXX  |			8
	.byte %01111100;| XXXXX  |			8
	.byte %01111100;| XXXXX  |			8
	.byte %01111100;| XXXXX  |			8

Level0
	.byte %00011111;|   XXXXX|
	.byte %00010000;|   X    |
	.byte %00010000;|   X    |
	.byte %00010000;|   X    |
	.byte %00010000;|   X    |
	
Level1
	.byte %01111100;| XXXXX  |
	.byte %01000000;| X      |
	.byte %01111001;| XXXX  X|
	.byte %01000001;| X     X|
	.byte %01111101;| XXXXX X|
	
Level2
	.byte %01000111;| X   XXX|
	.byte %10100100;|X X  X  |
	.byte %00010111;|   X XXX|
	.byte %00010100;|   X X  |
	.byte %00010111;|   X XXX|
	
Level3
	.byte %11011111;|XX XXXXX|
	.byte %00010000;|   X    |
	.byte %10010000;|X  X    |
	.byte %00010000;|   X    |
	.byte %11010000;|XX X    |
	
GameOver0
	.byte %00001110;|    XXX |
	.byte %00010010;|   X  X |
	.byte %00010110;|   X XX |
	.byte %00010000;|   X    |
	.byte %00001110;|    XXX |
GameOver1
	.byte %10010100;|X  X X  |
	.byte %10010100;|X  X X  |
	.byte %11110101;|XXXX X X|
	.byte %10010110;|X  X XX |
	.byte %01100100;| XX  X  |
GameOver2
	.byte %01011110;| X XXXX |
	.byte %01010000;| X X    |
	.byte %01011100;| X XXX  |
	.byte %11010000;|XX X    |
	.byte %01011110;| X XXXX |
GameOver3
	.byte %00110000;|  XX    |
	.byte %01001001;| X  X  X|
	.byte %01001010;| X  X X |
	.byte %01001010;| X  X X |
	.byte %00110010;|  XX  X |
GameOver4
	.byte %10001111;|X   XXXX|
	.byte %01001000;| X  X   |
	.byte %00101110;|  X XXX |
	.byte %00101000;|  X X   |
	.byte %00101111;|  X XXXX|
GameOver5
	.byte %01001000;| X  X   |
	.byte %01001000;| X  X   |
	.byte %01110000;| XXX    |
	.byte %01001000;| X  X   |
	.byte %01110000;| XXX    |
	
	align 256
;----------------------------------------------------------------------DrawIt
;
; Draw the values of the graphicsPointers. This routine was ripped from
; River Raid. It's a little different from the other 6 digit displays since
; it does a HMOVE on each scan line to blank the HMOVE bars.
;
; This routine MUST NOT cross a page boundary.
;
DrawIt	SUBROUTINE
	ldy fontHeight
.drawGraphicsLoop
	lda (graphicsPointers+8),y			;5
	tax										;2
	lda (graphicsPointers+10),y		;5
	sta WSYNC
;--------------------------
	sta HMOVE								;3
	sty fontHeight							;3
	sta temp									;3
	lda (graphicsPointers),y			;5
	sta GRP0									;3
	lda (graphicsPointers+2),y			;5		@ end of HBLANK
	sta GRP1									;3
	lda (graphicsPointers+4),y			;5
	sta GRP0									;3
	lda (graphicsPointers+6),y			;5
	ldy temp									;3

	sta GRP1									;3
	stx GRP0									;3
	sty GRP1									;3
	sta GRP0									;3
	ldy fontHeight							;3
	dey										;2
	bpl .drawGraphicsLoop				;2	@ 60 or 61
	
	iny										;62
	sty GRP0
	sty GRP1
	sty GRP0

	sta WSYNC
;--------------------------
	sta HMOVE
	rts

	org $FDF0
	
;---------------------------------------------------------HorizontalMoveTable
;
; This table holds HMPx values to adjust the player for fine horizonatal
; movement. This comes from a combination of Manuel Polik and Eric Ball.
;
; This table should be placed at the end of a page boundary to avoid masking
; the upper nibbles.
;
HorizontalMoveTable
	.byte HMOVE_L7
	.byte HMOVE_L6
	.byte HMOVE_L5
	.byte HMOVE_L4
	.byte HMOVE_L3
	.byte HMOVE_L2
	.byte HMOVE_L1
	.byte HMOVE_0
	.byte HMOVE_R1
	.byte HMOVE_R2
	.byte HMOVE_R3
	.byte HMOVE_R4
	.byte HMOVE_R5
	.byte HMOVE_R6
	.byte HMOVE_R7
	.byte HMOVE_R8
	
NumberFonts
   .byte $3C; |  XXXX  |
   .byte $66; | XX  XX |
   .byte $66; | XX  XX |
   .byte $66; | XX  XX |
   .byte $3C; |  XXXX  |
   
   .byte $3C; |  XXXX  |
   .byte $18; |   XX   |
   .byte $18; |   XX   |
   .byte $38; |  XXX   |
   .byte $18; |   XX   |
   
   .byte $7E; | XXXXXX |
   .byte $60; | XX     |
   .byte $3C; |  XXXX  |
   .byte $06; |     XX |
   .byte $7C; | XXXXX  |
   
   .byte $3C; |  XXXX  |
   .byte $46; | X   XX |
   .byte $0C; |    XX  |
   .byte $46; | X   XX |
   .byte $3C; |  XXXX  |
   
   .byte $0C; |    XX  |
   .byte $0C; |    XX  |
   .byte $7E; | XXXXXX |   
   .byte $4C; | X  XX  |
   .byte $4C; | X  XX  |
   
   .byte $7C; | XXXXX  |
   .byte $06; |     XX |
   .byte $7C; | XXXXX  |
   .byte $60; | XX     |
   .byte $7E; | XXXXXX |
   
   .byte $3C; |  XXXX  |
   .byte $66; | XX  XX |
   .byte $7C; | XXXXX  |
   .byte $60; | XX     |
   .byte $3C; |  XXXX  |
   
   .byte $18; |   XX   |
   .byte $18; |   XX   |
   .byte $0C; |    XX  |
   .byte $06; |     XX |
   .byte $7E; | XXXXXX |
   
   .byte $3C; |  XXXX  |
   .byte $66; | XX  XX |
   .byte $3C; |  XXXX  |
   .byte $66; | XX  XX |
   .byte $3C; |  XXXX  |
   
   .byte $3C; |  XXXX  |
   .byte $06; |     XX |   
   .byte $3E; |  XXXXX |
   .byte $66; | XX  XX |
   .byte $3C; |  XXXX  |
   
Blank
	.byte $00
	.byte $00
	.byte $00
	.byte $00
	.byte $00
	.byte $00
	.byte $00
	.byte $00
	.byte $00
	.byte $00
	.byte $00
	.byte $00
	.byte $00
	.byte $00

	.org $FFFC
	.word Start
	.word Start